/*************************************************
* DES Header File                                *
* (C) 1999-2001 The OpenCL Project               *
*************************************************/

#ifndef OPENCL_DES_H__
#define OPENCL_DES_H__

#include "opencl.h"
#include "triple.h"

namespace OpenCL {

class DES : public BlockCipher
   {
   public:
      static const u32bit BLOCKSIZE = 8, KEYLENGTH = 8;
      void encrypt(const byte[BLOCKSIZE], byte[BLOCKSIZE]) const;
      void decrypt(const byte[BLOCKSIZE], byte[BLOCKSIZE]) const;
      void encrypt(byte block[BLOCKSIZE]) const { encrypt(block, block); }
      void decrypt(byte block[BLOCKSIZE]) const { decrypt(block, block); }
      void set_key(const byte[], u32bit = KEYLENGTH) throw(InvalidKeyLength);
      void clear() throw() { round_key.clear(); }
   private:
      static const u32bit SPBOX1[64], SPBOX2[64], SPBOX3[64], SPBOX4[64],
                          SPBOX5[64], SPBOX6[64], SPBOX7[64], SPBOX8[64];
      void IP(u32bit&, u32bit&) const;
      void FP(u32bit&, u32bit&) const;
      void mask(u32bit&, u32bit&, byte, u32bit) const;
      void round(u32bit&, u32bit, u32bit) const;
      SecureBuffer<u32bit, 32> round_key;
   };

class DESX : public BlockCipher
   {
   public:
      static const u32bit BLOCKSIZE = 8, KEYLENGTH = 24;
      void encrypt(const byte[BLOCKSIZE], byte[BLOCKSIZE]) const;
      void decrypt(const byte[BLOCKSIZE], byte[BLOCKSIZE]) const;
      void encrypt(byte block[BLOCKSIZE]) const { encrypt(block, block); }
      void decrypt(byte block[BLOCKSIZE]) const { decrypt(block, block); }
      void set_key(const byte[], u32bit = KEYLENGTH) throw(InvalidKeyLength);
      void clear() throw() { des.clear(); K1.clear(); K2.clear(); }
   private:
      SecureBuffer<byte, BLOCKSIZE> K1, K2;
      DES des;
   };

typedef Triple<DES> TripleDES;

}

#endif
